package data

import (
	"time"

	"gorm.io/driver/mysql"

	"demo/internal/conf"

	"github.com/go-kratos/kratos/v2/log"
	"github.com/google/wire"
	"github.com/sirupsen/logrus"
	"gorm.io/gorm"
	"gorm.io/gorm/logger"
)

// ProviderSet is data providers.
var ProviderSet = wire.NewSet(NewData, NewReportRepo)

// Data .
type Data struct {
	db *gorm.DB
}

// NewData .
func NewData(c *conf.Data, logger log.Logger) (*Data, func(), error) {
	cleanup := func() {
		log.NewHelper(logger).Info("closing the data resources")
	}
	db, err := gorm.Open(mysql.New(mysql.Config{
		DriverName: c.Database.Driver,
		DSN:        c.Database.Source,
	}), &gorm.Config{
		Logger: NewGormLogger(),
	})
	if err != nil {
		return nil, cleanup, err
	}
	return &Data{db: db}, cleanup, nil
}

func NewGormLogger() logger.Interface {
	return logger.New(logrus.StandardLogger(), logger.Config{
		SlowThreshold:             time.Second,
		IgnoreRecordNotFoundError: true,
		LogLevel:                  logger.Info,
	})
}
